/*
 * This file is part of PRO CFW.

 * PRO CFW is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * PRO CFW is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with PRO CFW. If not, see <http://www.gnu.org/licenses/ .
 */

#include <pspsdk.h>
#include <pspkernel.h>
#include <pspsysmem_kernel.h>
#include <pspthreadman_kernel.h>
#include <pspdebug.h>
#include <pspinit.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <stdarg.h>
#include <pspctrl.h>
#include "systemctrl.h"


PSP_MODULE_INFO("analog_block", 0x1000, 0, 1);
static STMOD_HANDLER previous;


struct CtrlHookImports {
	u32 nid;
	void *fp;
};

typedef struct
{
	const char *name;
	unsigned short version;
	unsigned short attribute;
	unsigned char entLen;
	unsigned char varCount;
	unsigned short funcCount;
	unsigned int *fnids;
	unsigned int *funcs;
	unsigned int *vnids;
	unsigned int *vars;
}PspModuleImport;


#define MAKE_JUMP(f) (0x08000000 | (((u32)(f) >> 2) & 0x03ffffff))
#define MAKE_CALL(f) (0x0C000000 | (((u32)(f) >> 2) & 0x03ffffff))
#define MAKE_SYSCALL(n) ((n<<6)|12)
#define NOP 0

#define NELEMS(a) (sizeof(a) / sizeof(a[0]))
#define PatchSyscall sctrlHENPatchSyscall

static void remove_analog_input(SceCtrlData *data)
{
	if(data == NULL)
		return;

	data->Lx = 0xFF/2;
	data->Ly = 0xFF/2;
}

int myCtrlPeekBufferPositive(SceCtrlData *pad_data, int count)
{
	int ret;

	ret = sceCtrlPeekBufferPositive(pad_data, count);

	if(ret >= 0) {
		remove_analog_input(pad_data);
	}

	return ret;
}

int myCtrlPeekBufferNegative(SceCtrlData *pad_data, int count)
{
	int ret;

	ret = sceCtrlPeekBufferNegative(pad_data, count);

	if(ret >= 0) {
		remove_analog_input(pad_data);
	}

	return ret;
}

int myCtrlReadBufferPositive(SceCtrlData *pad_data, int count)
{
	int ret;

	ret = sceCtrlReadBufferPositive(pad_data, count);

	if(ret >= 0) {
		remove_analog_input(pad_data);
	}

	return ret;
}

int myCtrlReadBufferNegative(SceCtrlData *pad_data, int count)
{
	int ret;

	ret = sceCtrlReadBufferNegative(pad_data, count);

	if(ret >= 0) {
		remove_analog_input(pad_data);
	}

	return ret;
}

static struct CtrlHookImports g_ctrl_hooks[] = {
	{ 0x3A622550, &myCtrlPeekBufferPositive, },
	{ 0xC152080A, &myCtrlPeekBufferNegative, },
	{ 0x1F803938, &myCtrlReadBufferPositive, },
	{ 0x60B81F86, &myCtrlReadBufferNegative, },
};

static void patch_analog_imports()
{
	int i;

	for(i=0; i<NELEMS(g_ctrl_hooks); ++i) {
		PatchSyscall(FindProc("sceController_Service", "sceCtrl", g_ctrl_hooks[i].nid ), g_ctrl_hooks[i].fp);
	}
}


int module_start(SceSize args, void *argp)
{

	patch_analog_imports();
	return 0;
}
int module_stop( void )
{
  return 0;
}